class ContentType:
    """Word set content types."""

    WORD_SET = 'application/vnd.iris.ai.word-set+json'

    WORD_SET_ANALYSIS = 'application/vnd.iris.ai.word-set-analysis+json'

    WORD_SET_WORD = 'application/vnd.iris.ai.word+json'
