class ContentType:
    """AI Content type class."""

    ANALYSIS = 'application/vnd.iris.ai.analysis-request+json'

    ANALYSIS_FEEDBACK = 'application/vnd.iris.ai.analysis-feedback+json'

    JSON_DOCUMENT = 'application/json'

    CONFUSION_MATRIX = 'application/vnd.iris.ai.confusion-matrix+json'
