class UriTemplates:
    """Blip AI URI templates."""

    ANALYSIS = '/analysis'

    ANALYSIS_FEEDBACK = '/analysis/{0}/feedback'

    ANALYSES_FEEDBACK = '/analysis/feedback'

    ANALYSIS_EMAIL = '/enhancement/send-by-email'

    ANALYTICS = '/analytics/confusion-matrix'

    ANALYTICS_ID = '/analytics/confusion-matrix/{0}'
