class UriTemplates:
    """Entities uri templates."""

    INTENTIONS = '/intentions'

    INTENTION = '/intentions/{0}'

    INTENTION_ANSWERS = '/intentions/{0}/answers'

    INTENTION_ANSWER = '/intentions/{0}/answers/{1}'

    INTENTION_QUESTIONS = '/intentions/{0}/questions'

    INTENTION_QUESTION = '/intentions/{0}/questions/{1}'
