class UriTemplates:
    """AI Model uri templates."""

    MODELS = '/models'

    MODEL = '/model/{0}'

    MODELS_SUMMARY = '/models/summary'

    LAST_TRAINED_OR_PUBLISH_MODEL = '/models/last-trained-or-published'
