class ContentType:
    """Content Assistant Content type class."""

    CONTENT_RESULT = 'application/vnd.iris.ai.content-result+json'

    CONTENT_COMBINATION = 'application/vnd.iris.ai.content-combination+json'

    ANALYSIS = 'application/vnd.iris.ai.analysis-request+json'
