class ContentType:
    """Intents content type."""

    INTENTION = 'application/vnd.iris.ai.intention+json'

    ANSWER = 'application/vnd.iris.ai.answer+json'

    QUESTION = 'application/vnd.iris.ai.question+json'
